@@ -29,27 +29,20 @@ typedef struct {
     char   *saved_cursor;		/* cursor cache ID */
 } TLS_SCACHE;
 
-#define TLS_SCACHE_FLAG_DEL_CURSOR	(1<<0)
+#define TLS_SCACHE_FLAG_DEL_SAVED_CURSOR	(1<<0)
 
 extern TLS_SCACHE *tls_scache_open(const char *, const char *, int, int);
 extern void tls_scache_close(TLS_SCACHE *);
-extern int tls_scache_lookup(TLS_SCACHE *, const char *, long, int, long *, int *, VSTRING *);
-extern int tls_scache_update(TLS_SCACHE *, const char *, long, int, const char *, int);
+extern int tls_scache_lookup(TLS_SCACHE *, const char *, VSTRING *);
+extern int tls_scache_update(TLS_SCACHE *, const char *, const char *, int);
 extern int tls_scache_delete(TLS_SCACHE *, const char *);
-extern int tls_scache_sequence(TLS_SCACHE *, int, long, int, char **, long *, int *, VSTRING *);
-
-#define TLS_SCACHE_ANY_OPENSSL_VSN		((long) 0)
-#define TLS_SCACHE_ANY_FLAGS			(0)
+extern int tls_scache_sequence(TLS_SCACHE *, int, char **, VSTRING *);
 
 #define TLS_SCACHE_DONT_NEED_CACHE_ID		((char **) 0)
-#define TLS_SCACHE_DONT_NEED_OPENSSL_VSN	((long *) 0)
-#define TLS_SCACHE_DONT_NEED_FLAGS		((int *) 0)
 #define TLS_SCACHE_DONT_NEED_SESSION		((VSTRING *) 0)
 
 #define TLS_SCACHE_SEQUENCE_NOTHING \
-	TLS_SCACHE_ANY_FLAGS, TLS_SCACHE_ANY_OPENSSL_VSN, \
-	TLS_SCACHE_DONT_NEED_CACHE_ID, TLS_SCACHE_DONT_NEED_OPENSSL_VSN, \
-	TLS_SCACHE_DONT_NEED_FLAGS, TLS_SCACHE_DONT_NEED_SESSION
+	TLS_SCACHE_DONT_NEED_CACHE_ID, TLS_SCACHE_DONT_NEED_SESSION
 
 /* LICENSE
 /* .ad
