@@ -6,10 +6,10 @@
 /* SYNOPSIS
 /*	#include <iostuff.h>
 /*
-/*	ssize_t	dummy_read(fd, buf, buf_len, timeout, context)
+/*	int	dummy_read(fd, buf, buf_len, timeout, context)
 /*	int	fd;
 /*	void	*buf;
-/*	size_t	len;
+/*	unsigned len;
 /*	int	timeout;
 /*	void	*context;
 /* DESCRIPTION
@@ -52,10 +52,10 @@
 
 /* dummy_read - dummy read operation */
 
-ssize_t dummy_read(int fd, void *unused_buf, size_t len,
+int     dummy_read(int fd, void *unused_buf, unsigned len,
 		           int unused_timeout, void *unused_context)
 {
     if (msg_verbose)
-	msg_info("dummy_read: fd %d, len %lu", fd, (unsigned long) len);
+	msg_info("dummy_read: fd %d, len %d", fd, len);
     return (0);
 }
